package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microservice.goods.facade.clients.GoodsBatchManagerService;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchFeedbackForm;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchGradeForm;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchInfoForm;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordForm;
import com.hcloud.microservice.goods.service.GoodsBatchFeedbackService;
import com.hcloud.microservice.goods.service.GoodsBatchGradeService;
import com.hcloud.microservice.goods.service.GoodsBatchInfoService;
import com.hcloud.microservice.goods.service.GoodsQRCodeRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author JYJ
 */
@RestController
@RequestMapping("/goods/batch")
public class GoodsBatchManageController extends BaseController implements GoodsBatchManagerService {

    @Autowired
    private GoodsBatchInfoService goodsBatchInfoService;

    @Autowired
    private GoodsQRCodeRecordService goodsQRCodeRecordService;

    @Autowired
    private GoodsBatchFeedbackService goodsBatchFeedbackService;

    @Autowired
    private GoodsBatchGradeService goodsBatchGradeService;

    /**
     * 分页查询商品批次记录
     *
     * @param goodsBatchInfoForm
     * @return
     */
    @PostMapping("/queryGoodsBatchRecordByPage")
    @AuthOrgValidate
    @Override
    public ResultVO queryGoodsBatchRecordByPage(@RequestBody GoodsBatchInfoForm goodsBatchInfoForm) {
        PageInfo pageInfo = goodsBatchInfoService.searchGoodsBatchInfoByPage(goodsBatchInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 添加商品批次记录
     *
     * @param goodsBatchInfoForm
     * @return
     */
    @PostMapping("/addGoodsBatchRecord")
    @AuthOrgValidate
    @Override
    public ResultVO addGoodsBatchRecord(@RequestBody GoodsBatchInfoForm goodsBatchInfoForm) {
        GoodsBatchInfoForm queryForm = new GoodsBatchInfoForm();
        queryForm.setFkOrgGuid(goodsBatchInfoForm.getFkOrgGuid());
        queryForm.setGoodsBatchNo(goodsBatchInfoForm.getGoodsBatchNo());
        queryForm.setFkGoodsGuid(goodsBatchInfoForm.getFkGoodsGuid());
        PageInfo pageInfo = goodsBatchInfoService.searchGoodsBatchInfoByPage(queryForm);
        if(pageInfo.getTotal()>0){
            return new ResultVO(ResultEnum.CODE_BATCH_DUPLICATE);
        }
        int i = goodsBatchInfoService.addGoodsBatchInfo(goodsBatchInfoForm);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 编辑商品批次记录
     *
     * @param goodsBatchInfoForm
     * @return
     */
    @PostMapping("/editGoodsBatchRecord")
    @AuthOrgValidate
    @Override
    public ResultVO editGoodsBatchRecord(@RequestBody GoodsBatchInfoForm goodsBatchInfoForm) {
        int i = goodsBatchInfoService.editGoodsBatchInfo(goodsBatchInfoForm);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 查询商品批次记录明细
     *
     * @param guid
     * @return
     */
    @GetMapping("/getGoodsBatchRecordById")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsBatchRecordById(@RequestParam("id") String guid) {
        GoodsBatchInfoForm batchInfoForm = goodsBatchInfoService.searchGoodsBatchInfoById(guid);
        return success(batchInfoForm);
    }

    /**
     * 删除商品批次记录
     *
     * @param guid
     * @return
     */
    @GetMapping("/delGoodsBatchRecordById")
    @AuthOrgValidate
    @Override
    public ResultVO delGoodsBatchRecordById(@RequestParam("id") String guid,
                                            @RequestParam("orgGuid") String orgGuid) {
        GoodsBatchInfoForm batchInfoForm = new GoodsBatchInfoForm();
        batchInfoForm.setGuid(guid);
        batchInfoForm.setFkOrgGuid(orgGuid);
        int i = goodsBatchInfoService.deleteGoodsBatchInfo(batchInfoForm);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 生成根据产品批次信息生成溯源码记录
     *
     * @param goodsBatchInfoForm
     * @return
     */
    @PostMapping("/batchGoodsQRCode")
    @AuthOrgValidate
    @Override
    public ResultVO batchGoodsQRCode(@RequestBody GoodsBatchInfoForm goodsBatchInfoForm) {
        //TODO 生成二维码生成记录
        GoodsQRCodeRecordForm  recordForm = new GoodsQRCodeRecordForm();
        BeanCopyUtil.copyBean(goodsBatchInfoForm,recordForm);
        recordForm.setFkGoodsBatchGuid(goodsBatchInfoForm.getGuid());
        recordForm.setCodeCount(goodsBatchInfoForm.getQrcodeCount());
        int i = goodsQRCodeRecordService.generateGoodsQRCodeInfo(recordForm);
        if(i==0){
            return failure();
        }
        return success();
    }

    /**
     * 保存评价及评级信息
     *
     * @param goodsBatchInfoForm
     * @return
     */
    @PostMapping("/saveFeedback")
    @Override
    public ResultVO saveAllFeedbackInfo(@RequestBody GoodsBatchInfoForm goodsBatchInfoForm) {
        int i = goodsBatchInfoService.saveAllFeedbackInfo(goodsBatchInfoForm);
        if(i==0){failure();}
        return success();
    }

    /**
     * 查询用户商品批次评价信息
     *
     * @param batchId
     * @param userId
     * @return
     */
    @GetMapping("/getAllFeedback")
    @Override
    public ResultVO searchAllFeedbackInfo(@RequestParam("batchId") String batchId,@RequestParam("userId") String userId) {
        GoodsBatchInfoForm resultInfo = new GoodsBatchInfoForm();
        List<GoodsBatchFeedbackForm> feedbackList = goodsBatchFeedbackService.getGoodsFeedbackByUserId(userId,batchId);
        List<GoodsBatchGradeForm> gradeFormList = goodsBatchGradeService.getBatchGradeInfoByCond(batchId,userId);
        if(null!=feedbackList && feedbackList.size()>0){
            resultInfo.setIsFeedback(1);
            resultInfo.setFeedbackForm(feedbackList.get(0));
        }
        if(null!=gradeFormList && gradeFormList.size()>0){
            resultInfo.setIsGrade(1);
            resultInfo.setGradeForm(gradeFormList.get(0));
        }
       return success(resultInfo);
    }
}
