package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.domain.FileDO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.FileType;
import com.hcloud.microserver.commoncore.util.FileUtil;
import com.hcloud.microservice.goods.facade.clients.GoodsFieldManageService;
import com.hcloud.microservice.goods.facade.forms.GoodsFieldForm;
import com.hcloud.microservice.goods.service.GoodsFieldService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/goods/field")
public class GoodsFieldManageController extends BaseController implements GoodsFieldManageService {


    @Autowired
    private GoodsFieldService goodsFieldService;

    /**
     * 分页查询产地信息
     *
     * @param fieldForm
     * @return
     */
    @PostMapping("/searchByPage")
    @AuthOrgValidate
    @Override
    public ResultVO queryGoodsFieldByPage(@RequestBody GoodsFieldForm fieldForm) {
       PageInfo pageInfo =  goodsFieldService.searchGoodsFieldByPage(fieldForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 添加商品产地信息
     *
     * @param fieldForm
     * @return
     */
    @PostMapping("/addGoodsField")
    @AuthOrgValidate
    @Override
    public ResultVO addGoodsField(@RequestBody GoodsFieldForm fieldForm) {

        int i = goodsFieldService.addGoodsFieldInfo(fieldForm);
        if (i == 0) {
            return new ResultVO(ResultEnum.SYS_ERROR);
        }
        return success();
    }

    /**
     * 编辑商品产地信息
     *
     * @param fieldForm
     * @return
     */
    @PostMapping("/editGoodsField")
    @AuthOrgValidate
    @Override
    public ResultVO editGoodsField(@RequestBody GoodsFieldForm fieldForm) {
        int i = goodsFieldService.updateGoodsFieldInfo(fieldForm);
        if (i == 0) {
            return new ResultVO(ResultEnum.SYS_ERROR);
        }
        return success();
    }

    /**
     * 获取商品产地明细信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/getGoodsField")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsField(@RequestParam("id") String guid,@RequestParam("orgId") String orgGuid) {
       GoodsFieldForm goodsFieldForm =  goodsFieldService.getGoodsFieldInfoById(guid,orgGuid);
       if(null!= goodsFieldForm){
           return success(goodsFieldForm);
       }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    /**
     * 删除商品产地信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/delGoodsField")
    @AuthOrgValidate
    @Override
    public ResultVO delGoodsField(@RequestParam("id") String guid,@RequestParam("orgId") String orgGuid) {
       int i= goodsFieldService.delByPrimaryKey(guid);
       if (i == 0) {
            return new ResultVO(ResultEnum.SYS_ERROR); }
        return success();
    }


}
