package com.hcloud.microservice.goods.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microservice.goods.facade.clients.GoodsManagerService;
import com.hcloud.microservice.goods.facade.entity.GoodsInfo;
import com.hcloud.microservice.goods.facade.forms.GoodsInfoForm;
import com.hcloud.microservice.goods.service.GoodsInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/goods/info")
public class GoodsManageController extends BaseController implements GoodsManagerService {

    @Autowired
    private GoodsInfoService goodsInfoService;

    /**
     * 分页查询商品信息
     *
     * @param goodsInfoForm
     * @return
     */
    @PostMapping("/searchByPage")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsInfoByPage(@RequestBody GoodsInfoForm goodsInfoForm) {
       PageInfo pageInfo =  goodsInfoService.searchGoodsInfoByPage(goodsInfoForm);
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    /**
     * 添加产品信息
     *
     * @param goodsInfoForm
     * @return
     */
    @PostMapping("/addGoodsInfo")
    @AuthOrgValidate
    @Override
    public ResultVO addGoodsInfo(@RequestBody GoodsInfoForm goodsInfoForm) {
        GoodsInfo goodsInfo = new GoodsInfo();
        BeanCopyUtil.copyBean(goodsInfoForm,goodsInfo);
        int i = goodsInfoService.saveSelective(goodsInfo);
        if(i==0){
            return new ResultVO(ResultEnum.SYS_ERROR);
        }
        return success();
    }

    /**
     * 编辑产品信息
     *
     * @param goodsInfoForm
     * @return
     */
    @PostMapping("/editGoodsInfo")
    @AuthOrgValidate
    @Override
    public ResultVO editGoodsInfo(@RequestBody GoodsInfoForm goodsInfoForm) {
        GoodsInfo goodsInfo = new GoodsInfo();
        BeanCopyUtil.copyBean(goodsInfoForm,goodsInfo);
        int i = goodsInfoService.modifyByPrimaryKeySelective(goodsInfo);
        if(i==0){
            return new ResultVO(ResultEnum.SYS_ERROR);
        }
        return success();
    }

    /**
     * 通过Id获取产品信息明细
     *
     * @param guid
     * @return
     */
    @GetMapping("/getGoodsInfoById")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsInfoById(@RequestParam("id") String guid) {
       GoodsInfoForm goodsInfoForm =  goodsInfoService.searchGoodsInfoById(guid);
        return success(goodsInfoForm);
    }

    /**
     * 删除产品信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/delGoodsInfoById")
    @Override
    public ResultVO delGoodsInfoById(@RequestParam("id") String guid) {
        GoodsInfo goodsInfo = new GoodsInfo();
        goodsInfo.setGuid(guid);
        goodsInfo.setState(0);
        goodsInfo.setModifiedTime(new Date());
       int i = goodsInfoService.modifyByPrimaryKeySelective(goodsInfo);
       if(i==0){return failure();}
        return success();
    }
}
