package com.hcloud.microservice.goods.controller;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microservice.goods.facade.clients.GoodsQRCodeManagerService;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeInfo;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeInfoForm;
import com.hcloud.microservice.goods.service.GoodsQRCodeInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/qrcode/info")
public class GoodsQRCodeInfoController extends BaseController implements GoodsQRCodeManagerService {

    @Autowired
    private GoodsQRCodeInfoService goodsQRCodeInfoService;

    /**
     * 分页查询商品溯源码信息
     *
     * @param goodsQRCodeInfoForm
     * @return
     */
    @PostMapping("/searchByPage")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsQRCodeByPage(@RequestBody GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
        PageInfo pageInfo = goodsQRCodeInfoService.queryGoodsQRCodeInfoByRecId(goodsQRCodeInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 获取商品溯源码明细信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/getGoodsQRCodeById")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsQRCodeById(@RequestParam("id") String guid) {
        GoodsQRCodeInfo info = goodsQRCodeInfoService.queryByPrimaryKey(guid);
        return success(info);
    }

    /**
     * 删除商品溯源码信息
     *
     * @param goodsQRCodeInfoForm
     * @return
     */
    @PostMapping("/delGoodsQRCodeById")
    @AuthOrgValidate
    @Override
    public ResultVO delGoodsQRCodeById(@RequestBody GoodsQRCodeInfoForm goodsQRCodeInfoForm) {
       GoodsQRCodeInfo codeInfo =  goodsQRCodeInfoService.queryByPrimaryKey(goodsQRCodeInfoForm.getGuid());
       if(codeInfo.getState()!=0){
           int i = goodsQRCodeInfoService.forbiddenCodeInfoByCodeId(goodsQRCodeInfoForm);
           if(i==0){
               return failure();
           }
           return success();
       }
        return new ResultVO(ResultEnum.CODE_DEL_DUPLICATE);
    }

    /**
     * 查询导出溯源码信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/searchGoodsQRCodeForExport")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsQRCodeForExport(@RequestParam("id") String guid) {
        GoodsQRCodeInfo info = goodsQRCodeInfoService.queryByPrimaryKey(guid);
        return success(info);
    }

    /**
     * 通过二维码传输码查询溯源码信息
     *
     * @param tokenStr
     * @return
     */
    @GetMapping("/searchByToken")
    @AuthOrgValidate
    @Override
    public ResultVO searchGoodsQRCodeInfoByTokenStr(@RequestParam("token") String tokenStr) {
        int i =goodsQRCodeInfoService.updateGoodsQRInfoByScan(tokenStr);
        if(i!=0){
            GoodsQRCodeInfoForm codeInfoForm = goodsQRCodeInfoService.getGoodsQRInfo(tokenStr);
            return success(codeInfoForm);
        }
        return failure();
    }

    /**
     * 通过二维码传输码预览溯源码信息
     *
     * @param tokenStr
     * @return
     */
    @GetMapping("/getDetailByToken")
    @AuthOrgValidate
    @Override
    public ResultVO getGoodsQRCodeDetailInfoByToken(@RequestParam("token") String tokenStr) {
        GoodsQRCodeInfoForm codeInfoForm = goodsQRCodeInfoService.getGoodsQRInfo(tokenStr);
        return success(codeInfoForm);
    }
    
    @PostMapping("/getPrintDetail")
    @AuthOrgValidate
    @Override
    public ResultVO getPrintDetail(@RequestBody GoodsQRCodeInfoForm goodsQRCodeInfoForm){
    	PageInfo pageInfo = goodsQRCodeInfoService.getPrintGoodsQRInfo(goodsQRCodeInfoForm);
        return success(pageInfo);
    }

    @PostMapping("/updateQRIsPrint")
	@Override
	public ResultVO updateQRIsPrint(@RequestBody List<String> ids) {
		int i = goodsQRCodeInfoService.updateQRIsPrint(ids);
		if(i == 0) {
			return failure();
		}
		return success(i);
	}
}
