package com.hcloud.microservice.goods.dao;


import com.hcloud.microserver.commoncore.base.BaseMapper;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microservice.goods.bo.GoodsQRCodeRecordBO;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeRecord;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordForm;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordStatForm;
import com.hcloud.microservice.goods.facade.results.ScanNumStatResult;
import com.hcloud.microservice.goods.facade.results.ScanNumStatisResult;

import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author xiezt
 */
public interface GoodsQRCodeRecordMapper extends BaseMapper<GoodsQRCodeRecord,String> {

    /**
     * 分页查询
     * @param pageBean
     * @return
     */
    List<GoodsQRCodeRecordBO> searchByPage(PageBean<GoodsQRCodeRecordForm> pageBean);

    /**
     * 获取二维码生成父码最大值
     * @param codePrefix 父码前缀
     * @return
     */
    String getMaxParentCode(@Param("parentCode") String codePrefix);

    /**
     * 批量更新溯源码生成记录
     * @param list
     * @return
     */
    int batchUpdateGoodsQRCodeRecord(List<String> list);

    /**
     * 通过商品批次编号查询溯源码生成记录
     * @param batchId
     * @return
     */
    List<GoodsQRCodeRecord> getGoodsQRCodeRecordByBatchId(@Param("batchId") String batchId);

    /**
     * 统计全年各月的扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    ScanNumStatResult statQRCodeScanNumForOneYeaAllMonth(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);

    /**
     * 统计当天和历史累计扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    List<ScanNumStatResult> statYesterdayAndAllTime(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);

    /**
     * 统计一天24小时的扫码数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    List<ScanNumStatResult> statOneDayAllHours(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);

    /**
     *
     * 统计一周7天的扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    List<ScanNumStatResult> statOneWeekSevenDays(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);

    /**
     * 统计一个月每天的扫码人数
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    List<ScanNumStatResult> statOneMonthAllDays(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);
    
    /**
     * 统计个月扫码人数
     * @author lym
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    List<ScanNumStatisResult> statQRCodeScanNumByMonth(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm);
}