package com.hcloud.microservice.goods.eventlisteners;

import com.hcloud.microserver.commoncore.util.QrCodeUtil;
import com.hcloud.microservice.goods.enums.TracedCodeImgInfoEnum;
import com.hcloud.microservice.goods.event.TracedQRCodeImgEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class BusinessListener {

    @Value("${qr.code.path}")
    private String qrcodeImgPath;

    @Value("${qr.code.logo.path}")
    private String orgLogoDirectory;

    @Async
    @EventListener
    public void generateQRCodeWithLogo(TracedQRCodeImgEvent tracedQRCodeImgEvent){
        //获取生成二维码图片信息列表
        List<Map> tracedCodeList = tracedQRCodeImgEvent.getQrcodeInfoList();
        if(null!= tracedCodeList && tracedCodeList.size()>0){
            QrCodeUtil qrCodeUtil = new QrCodeUtil();
            for(Map map : tracedCodeList){
                String imgSubPathName = (String) map.get(TracedCodeImgInfoEnum.IMG_SUB_DIR_NAME);
                String logoName = (String) map.get(TracedCodeImgInfoEnum.ORG_LOGO_NAME);
                String codeContent = (String) map.get(TracedCodeImgInfoEnum.CODE_CONTENT);
                String imgName = (String) map.get(TracedCodeImgInfoEnum.IMG_NAME);
                String orgLogoPath= orgLogoDirectory+logoName;
                String imgPath = qrcodeImgPath+imgSubPathName;
                try {
                    log.info("codeContent==============>{}",codeContent);
                    log.info("orgLogoPath==============>{}",orgLogoPath);
                    log.info("imgPath==============>{}",imgPath);
                    File filePath = new File(orgLogoPath);
                    if(!filePath.exists()){
                        qrCodeUtil.generateQRCode(codeContent,imgPath,imgName,"png");
                    }else{
                        qrCodeUtil.generateQRCodeWithLogo(codeContent,orgLogoPath,imgPath,imgName,"png");
                    }
                } catch (Exception e) {
                    log.info("generate qr code error  =============>{}",e.getMessage());
                    e.printStackTrace();
                }
            }
        }
    }
}
