package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microservice.goods.bo.GoodsQRCodeInfoBO;
import com.hcloud.microservice.goods.bo.GoodsQRCodeRecordBO;
import com.hcloud.microservice.goods.dao.GoodsBatchInfoMapper;
import com.hcloud.microservice.goods.dao.GoodsQRCodeRecordMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsBatchInfo;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeInfo;
import com.hcloud.microservice.goods.facade.entity.GoodsQRCodeRecord;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeInfoForm;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordForm;
import com.hcloud.microservice.goods.service.GoodsQRCodeInfoService;
import com.hcloud.microservice.goods.service.GoodsQRCodeRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author apple
 */
@Transactional(rollbackFor = Exception.class)
@Service("goodsQRCodeRecordService")
public class GoodQRCodeRecordServiceImpl implements GoodsQRCodeRecordService {

    /**
     * 样本数据
     */
    private static final Integer SAMPLE_TEN_VALUE=10;

    private static final Integer SAMPLE_HUNDRED_VALUE=100;

    @Resource
    private GoodsQRCodeRecordMapper goodsQRCodeRecordMapper;

    @Resource
    private GoodsBatchInfoMapper goodsBatchInfoMapper;

    @Autowired
    private GoodsQRCodeInfoService goodsQRCodeInfoService;

    /**
     * 分页查询二维码生成记录
     *
     * @param recordForm
     * @return
     */
    @Override
    public PageInfo searchQRCodeRecordByPage(GoodsQRCodeRecordForm recordForm) {
        PageBean pageBean = new PageBean(recordForm);
        PageHelper.startPage(recordForm.getPageNo(),recordForm.getPageSize());
        List<GoodsQRCodeRecordBO> list = goodsQRCodeRecordMapper.searchByPage(pageBean);
        return new PageInfo(list);
    }

    /**
     * 生成溯源二维码记录（含溯源码明细、包含溯源码信息、溯源码图片）
     *
     * @param goodsQRCodeRecordForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int generateGoodsQRCodeInfo(GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        if(null!=goodsQRCodeRecordForm && !"".equals(goodsQRCodeRecordForm.getFkGoodsBatchGuid())){
            //TODO 产生二维码生成记录数据
            GoodsQRCodeRecord goodsQRCodeRecord = new GoodsQRCodeRecord();
            BeanCopyUtil.copyBean(goodsQRCodeRecordForm,goodsQRCodeRecord);
            String orgBrevity = goodsQRCodeRecordForm.getOrgBrevity();
            String goodsBatchId = goodsQRCodeRecord.getFkGoodsBatchGuid();
            //生成二维码父码
            goodsQRCodeRecord.setGoodsParentQrcode(getMaxParentCode(orgBrevity,goodsBatchId));
            goodsQRCodeRecord.setQrcodeRecordTime(new Date());
            goodsQRCodeRecord.setOriginNum(goodsQRCodeRecordForm.getCodeCount());
            goodsQRCodeRecord.setCancelNum(0);
            goodsQRCodeRecord.setScanNum(0);
            goodsQRCodeRecord.setUsedNum(0);
            int i =this.saveSelective(goodsQRCodeRecord);
            if(i!=0){
                //TODO 产生二维码生成记录明细
                GoodsQRCodeInfoForm codeInfoForm = new GoodsQRCodeInfoForm();
                codeInfoForm.setCodeCount(goodsQRCodeRecordForm.getCodeCount());
                codeInfoForm.setFkOrgGuid(goodsQRCodeRecord.getFkOrgGuid());
                codeInfoForm.setFkBatchRecordGuid(goodsQRCodeRecord.getGuid());
                codeInfoForm.setQrcodeParentCode(goodsQRCodeRecord.getGoodsParentQrcode());
                codeInfoForm.setGoodsBatchInfoId(goodsBatchId);
                codeInfoForm.setOrgBrevity(orgBrevity);
                codeInfoForm.setIco(goodsQRCodeRecordForm.getIco());
                codeInfoForm.setCodeUrlPrefix(goodsQRCodeRecordForm.getCodeUrlPrefix());
                codeInfoForm.setCreateUser(goodsQRCodeRecordForm.getCreateUser());
                codeInfoForm.setModifiedUser(goodsQRCodeRecordForm.getModifiedUser());
                int insertCount = goodsQRCodeInfoService.batchSaveGoodsQRCodeInfo(codeInfoForm);
                return insertCount;
            }
        }
        return 0;
    }

    /**
     * 生成最大的二维码父码信息
     * @param orgBrevity 机构简码
     * @param guid 商品批次记录主键
     * @return
     */
    private String getMaxParentCode(String orgBrevity,String guid){
        StringBuilder stb = new StringBuilder();
        GoodsBatchInfo goodsBatchInfo = goodsBatchInfoMapper.selectByPrimaryKey(guid);
        if(goodsBatchInfo!=null){
            String batchNo = goodsBatchInfo.getGoodsBatchNo();
            String codePrefix = orgBrevity+batchNo;
            stb.append(codePrefix);
            String maxCode = goodsQRCodeRecordMapper.getMaxParentCode(codePrefix);
            if(null!=maxCode && !"".equals(maxCode)){
                Integer num = Integer.valueOf(maxCode.substring(codePrefix.length()));
                num++;
                if(num<SAMPLE_TEN_VALUE){
                    stb.append("00").append(num);
                }else if(num<SAMPLE_HUNDRED_VALUE){
                    stb.append("0").append(num);
                }
            }else{
                stb.append("001");
            }
            return stb.toString();
        }
        return null;
    }

    /**
     * 获取二维码生成详情
     *
     * @param queryInfo
     * @return
     */
    @Override
    public PageInfo getGoodsQRCodeDetailById(GoodsQRCodeInfoForm queryInfo) {
        PageInfo pageInfo = goodsQRCodeInfoService.queryGoodsQRCodeInfoByRecId(queryInfo);
        List<GoodsQRCodeInfoForm> resultList = BeanCopyUtil.convertList(pageInfo.getList(),GoodsQRCodeInfoForm.class);
        pageInfo.setList(resultList);
        return pageInfo;
    }

    /**
     * 删除二维码生成记录
     * @param goodsQRCodeRecordForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int delGoodsQRCodeRecord(GoodsQRCodeRecordForm goodsQRCodeRecordForm) {
        //TODO 删除溯源码生成记录
        GoodsQRCodeRecord codeRecord = goodsQRCodeRecordMapper.selectByPrimaryKey(goodsQRCodeRecordForm.getGuid());
        codeRecord.setState(0);
        codeRecord.setCancelNum(codeRecord.getOriginNum());
        codeRecord.setModifiedUser(goodsQRCodeRecordForm.getModifiedUser());
        codeRecord.setModifiedTime(new Date());
        int i = goodsQRCodeRecordMapper.updateByPrimaryKeySelective(codeRecord);
        if (i==0){throw new RuntimeException();}
        //TODO 删除溯源码详细信息记录
        GoodsQRCodeInfoForm codeInfoForm = new GoodsQRCodeInfoForm();
        codeInfoForm.setFkBatchRecordGuid(goodsQRCodeRecordForm.getGuid());
        codeInfoForm.setState(0);
        codeInfoForm.setModifiedUser(goodsQRCodeRecordForm.getModifiedUser());
        codeInfoForm.setModifiedTime(new Date());
        int j = goodsQRCodeInfoService.delGoodsQRCodeInfoByRecordId(codeInfoForm);
        if(j==0){throw new RuntimeException();}
        //TODO 删除溯源码图片
        return 1;
    }

    /**
     * 删除二维码生成记录（物理删除）
     *
     * @param recordId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int delQRcodeInfoByPhysical(String recordId) {
        int i = goodsQRCodeRecordMapper.deleteByPrimaryKey(recordId);
        if (i==0){throw new RuntimeException();}
        goodsQRCodeInfoService.delQRCodeInfoByPhysical(recordId);
        return i;
    }

    /**
     * 批量删除溯源码生成记录（逻辑删除）
     *
     * @param batchId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int delGoodsQRCodeRecordByBatchId(String batchId) {
        if(batchId!=null && !"".equals(batchId)){
            List<String> recordIdList = Lists.newArrayList();
           List<GoodsQRCodeRecord> batchIdList =  goodsQRCodeRecordMapper.getGoodsQRCodeRecordByBatchId(batchId);
           if(null!=batchIdList && batchIdList.size()>0){
               for (GoodsQRCodeRecord bean:batchIdList) {
                   recordIdList.add(bean.getGuid());
               }
              //删除溯源码记录信息
               int i = goodsQRCodeRecordMapper.batchUpdateGoodsQRCodeRecord(recordIdList);
               if(i==0){throw new RuntimeException();}
              //删除溯源码明细记录
               int j =  goodsQRCodeInfoService.batchUpdateGoodsQRCodeInfoByRecord(recordIdList);
               if(j==0){throw new RuntimeException();}
               return j;
           }
           return 1;
        }
        return 0;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsQRCodeRecordMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsQRCodeRecord record) {
        record.setGuid(GetUUID.getUUID());
        return goodsQRCodeRecordMapper.insert(record);
    }

    @Override
    public int saveSelective(GoodsQRCodeRecord record) {
        record.setGuid(GetUUID.getUUID());
        return goodsQRCodeRecordMapper.insertSelective(record);
    }

    @Override
    public GoodsQRCodeRecord queryByPrimaryKey(String guid) {
        return goodsQRCodeRecordMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsQRCodeRecord record) {
        return goodsQRCodeRecordMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsQRCodeRecord record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsQRCodeRecord record) {
        return goodsQRCodeRecordMapper.updateByPrimaryKey(record);
    }
}
