package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microservice.goods.bo.GoodsFeedbackBO;
import com.hcloud.microservice.goods.dao.GoodsBatchFeedbackMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsBatchFeedback;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchFeedbackForm;
import com.hcloud.microservice.goods.service.GoodsBatchFeedbackService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author JYJ
 */
@Service("goodsBatchFeedbackService")
public class GoodsBatchFeedbackServiceImpl implements GoodsBatchFeedbackService {


    @Resource
    private GoodsBatchFeedbackMapper goodsBatchFeedbackMapper;

    /**
     * 分页查询评价列表
     *
     * @param feedbackForm
     * @return
     */
    @Override
    public PageInfo searchBatchFeedbackByPage(GoodsBatchFeedbackForm feedbackForm) {
       PageHelper.startPage(feedbackForm.getPageNo(),feedbackForm.getPageSize());
       List<GoodsFeedbackBO> list =  goodsBatchFeedbackMapper.searchBatchFeedbackByPage(new PageBean(feedbackForm));
       return new PageInfo(list);
    }

    /**
     * 通过溯源用户id,查询商品评价列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<GoodsBatchFeedbackForm> getGoodsFeedbackByUserId(String userId,String batchId) {
        List<GoodsBatchFeedback> list = goodsBatchFeedbackMapper.getGoodsFeedbackByUserId(userId,batchId);
        List<GoodsBatchFeedbackForm> resultList = BeanCopyUtil.convertList(list,GoodsBatchFeedbackForm.class);
        return resultList;
    }

    /**
     * 批量删除评价信息
     *
     * @param pkList
     * @return
     */
    @Override
    public int delGoodsFeedbackByList(List<String> pkList) {
        return goodsBatchFeedbackMapper.delGoodsFeedbackByList(pkList);
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsBatchFeedbackMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsBatchFeedbackForm record) {
        GoodsBatchFeedback batchFeedback = new GoodsBatchFeedback();
        BeanCopyUtil.copyBean(record,batchFeedback);
        batchFeedback.setGuid(UUIDUtils.randomUUID());
        return goodsBatchFeedbackMapper.insert(batchFeedback);
    }

    @Override
    public int saveSelective(GoodsBatchFeedbackForm record) {
        GoodsBatchFeedback batchFeedback = new GoodsBatchFeedback();
        BeanCopyUtil.copyBean(record,batchFeedback);
        batchFeedback.setGuid(UUIDUtils.randomUUID());
        return goodsBatchFeedbackMapper.insertSelective(batchFeedback);
    }

    @Override
    public GoodsBatchFeedbackForm queryByPrimaryKey(String guid) {
        GoodsBatchFeedbackForm feedbackForm = new GoodsBatchFeedbackForm();
        GoodsBatchFeedback goodsBatchFeedback = goodsBatchFeedbackMapper.selectByPrimaryKey(guid);
        BeanCopyUtil.copyBean(goodsBatchFeedback,feedbackForm);
        return feedbackForm;
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsBatchFeedbackForm record) {
        GoodsBatchFeedback batchFeedback = new GoodsBatchFeedback();
        BeanCopyUtil.copyBean(record,batchFeedback);
        return goodsBatchFeedbackMapper.updateByPrimaryKeySelective(batchFeedback);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsBatchFeedbackForm record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsBatchFeedbackForm record) {
        GoodsBatchFeedback batchFeedback = new GoodsBatchFeedback();
        BeanCopyUtil.copyBean(record,batchFeedback);
        return goodsBatchFeedbackMapper.updateByPrimaryKey(batchFeedback);
    }

}
