package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microservice.goods.bo.GoodsGradeBO;
import com.hcloud.microservice.goods.dao.GoodsBatchGradeMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsBatchGrade;
import com.hcloud.microservice.goods.facade.forms.GoodsBatchGradeForm;
import com.hcloud.microservice.goods.service.GoodsBatchGradeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author JYJ
 */
@Service("goodsBatchGradeService")
public class GoodsBatchGradeServiceImpl implements GoodsBatchGradeService {


    @Resource
    private GoodsBatchGradeMapper goodsBatchGradeMapper;

    /**
     * 分页查询评价星级
     *
     * @param gradeForm
     * @return
     */
    @Override
    public PageInfo searchBatchGradeByPage(GoodsBatchGradeForm gradeForm) {
        PageHelper.startPage(gradeForm.getPageNo(),gradeForm.getPageSize());
        List<GoodsGradeBO> resultList = goodsBatchGradeMapper.searchBatchGradeByPage(new PageBean(gradeForm));
        return new PageInfo(resultList);
    }

    /**
     * 获取商品评价信息列表
     * @param goodsBatchId
     * @param uid
     * @return
     */
    @Override
    public List<GoodsBatchGradeForm> getBatchGradeInfoByCond(String uid, String goodsBatchId) {
        List<GoodsBatchGrade> list =  goodsBatchGradeMapper.getBatchGradeInfoByCond(uid,goodsBatchId);
        List<GoodsBatchGradeForm> resultList = BeanCopyUtil.convertList(list,GoodsBatchGradeForm.class);
        return resultList;
    }

    /**
     * 批量删除评价星级信息
     * @param pkList
     */
    @Override
    public int delBatchGradeInfo(List<String> pkList) {
        return goodsBatchGradeMapper.delBatchGradeInfo(pkList);
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsBatchGradeMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsBatchGradeForm record) {
        GoodsBatchGrade batchGrade = new GoodsBatchGrade();
        BeanCopyUtil.copyBean(record,batchGrade);
        batchGrade.setGuid(UUIDUtils.randomUUID());
        return goodsBatchGradeMapper.insert(batchGrade);
    }

    @Override
    public int saveSelective(GoodsBatchGradeForm record) {
        GoodsBatchGrade batchGrade = new GoodsBatchGrade();
        BeanCopyUtil.copyBean(record,batchGrade);
        batchGrade.setGuid(UUIDUtils.randomUUID());
        return goodsBatchGradeMapper.insertSelective(batchGrade);
    }

    @Override
    public GoodsBatchGradeForm queryByPrimaryKey(String guid) {
        GoodsBatchGradeForm goodsBatchGradeForm = new GoodsBatchGradeForm();
        GoodsBatchGrade goodsBatchGrade = goodsBatchGradeMapper.selectByPrimaryKey(guid);
        BeanCopyUtil.copyBean(goodsBatchGrade,goodsBatchGradeForm);
        return goodsBatchGradeForm;
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsBatchGradeForm record) {
        GoodsBatchGrade batchGrade = new GoodsBatchGrade();
        BeanCopyUtil.copyBean(record,batchGrade);
        return goodsBatchGradeMapper.updateByPrimaryKeySelective(batchGrade);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsBatchGradeForm record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsBatchGradeForm record) {
        GoodsBatchGrade batchGrade = new GoodsBatchGrade();
        BeanCopyUtil.copyBean(record,batchGrade);
        return goodsBatchGradeMapper.updateByPrimaryKey(batchGrade);
    }
}
