package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.exception.GlobalException;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.DateUtils;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.string.ConvertStringToList;
import com.hcloud.microserver.commoncore.util.string.StringUtil;
import com.hcloud.microservice.goods.dao.GoodsBatchInfoMapper;
import com.hcloud.microservice.goods.dao.GoodsBatchLabelMapper;
import com.hcloud.microservice.goods.dao.GoodsInfoAuxiliaryMapper;
import com.hcloud.microservice.goods.dao.GoodsLabelMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsBatchInfo;
import com.hcloud.microservice.goods.facade.entity.GoodsBatchLabel;
import com.hcloud.microservice.goods.facade.entity.GoodsInfoAuxiliary;
import com.hcloud.microservice.goods.facade.entity.GoodsInterAuxiliary;
import com.hcloud.microservice.goods.facade.entity.GoodsLabel;
import com.hcloud.microservice.goods.facade.forms.*;
import com.hcloud.microservice.goods.service.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author JYJ
 */
@Transactional(rollbackFor = Exception.class)
@Service("goodsBatchInfoService")
public class GoodsBatchInfoServiceImpl implements GoodsBatchInfoService {
	
	private Logger log = LoggerFactory.getLogger(GoodsBatchInfoServiceImpl.class);

    private static final Integer PROCESS_SOURCE_TYPE=1;

    private static final Integer PROCESS_PRODUCT_TYPE=2;

    @Resource
    private GoodsBatchInfoMapper goodsBatchInfoMapper;

    @Resource
    private GoodsInfoAuxiliaryMapper goodsInfoAuxiliaryMapper;

    @Autowired
    private GoodsAssayInfoService goodsAssayInfoService;

    @Autowired
    private GoodsQRCodeRecordService goodsQRCodeRecordService;

    @Autowired
    private GoodsProInputsService goodsProInputsService;

    @Autowired
    private GoodsProNormalService goodsProNormalService;

    @Autowired
    private GoodsBatchFeedbackService goodsBatchFeedbackService;

    @Autowired
    private GoodsBatchGradeService goodsBatchGradeService;
    
    @Resource
    private GoodsLabelMapper goodsLabelMapper;
    
    @Resource
    private GoodsBatchLabelMapper goodsBatchLabelMapper;
    
    @Autowired
    private GoodsInterAuxiliaryService goodsInterAuxiliaryService;
    
    @Override
    public int delByPrimaryKey(String guid) {
        return goodsBatchInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsBatchInfo record) {
        record.setGuid(GetUUID.getUUID());
        return goodsBatchInfoMapper.insert(record);
    }

    @Override
    public int saveSelective(GoodsBatchInfo record) {
        record.setGuid(GetUUID.getUUID());
        return goodsBatchInfoMapper.insertSelective(record);
    }

    @Override
    public GoodsBatchInfo queryByPrimaryKey(String guid) {
        return goodsBatchInfoMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsBatchInfo record) {
        return goodsBatchInfoMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsBatchInfo record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsBatchInfo record) {
        return goodsBatchInfoMapper.updateByPrimaryKey(record);
    }

    /**
     * 分页查询商品批次信息
     *
     * @param batchInfoForm
     * @return
     */
    @Override
    public PageInfo searchGoodsBatchInfoByPage(GoodsBatchInfoForm batchInfoForm) {
        PageBean pageBean = new PageBean(batchInfoForm);
        PageHelper.startPage(batchInfoForm.getPageNo(),batchInfoForm.getPageSize());
        List list = goodsBatchInfoMapper.searchByPage(pageBean);
        return new PageInfo(list);
    }

    /**
     * 添加商品批次信息
     *
     * @param batchInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int addGoodsBatchInfo(GoodsBatchInfoForm batchInfoForm) {
        //TODO 添加商品批次信息
        if(null!=batchInfoForm){
            GoodsAssayInfoForm assayInfo = batchInfoForm.getAssayInfoForm();
            List<GoodsProInputsForm> inputsFormList = batchInfoForm.getInputsFormList();
            List<GoodsProNormalForm> materialList = batchInfoForm.getMaterialList();
            List<GoodsProNormalForm> productList = batchInfoForm.getProductList();
            GoodsBatchInfo batchInfo = new GoodsBatchInfo();
            BeanCopyUtil.copyBean(batchInfoForm,batchInfo);
            int i = this.saveSelective(batchInfo);
            //添加产品标签
            batchInfoForm.setGuid(batchInfo.getGuid());
            saveOrUpdateGoodsBatchLabel(batchInfoForm);
            //添加辅助项信息
            List<GoodsInfoAuxiliaryForm> auxList = batchInfoForm.getAuxList();
            if(null!=auxList && auxList.size()>0){
                for (GoodsInfoAuxiliaryForm bean :auxList) {
                    bean.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                    bean.setFkAuxItmeId(batchInfo.getGuid());
                    bean.setFkGoodsGuid(batchInfoForm.getFkGoodsGuid());
                }
                goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
            }
            //添加检验信息
            if(assayInfo!=null){
                assayInfo.setFkGoodsBatchId(batchInfo.getGuid());
                assayInfo.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                assayInfo.setState(batchInfoForm.getState());
                assayInfo.setCreateUser(batchInfoForm.getCreateUser());
                assayInfo.setCreateTime(batchInfoForm.getCreateTime());
                assayInfo.setModifiedUser(batchInfoForm.getModifiedUser());
                assayInfo.setModifiedTime(batchInfoForm.getModifiedTime());
                goodsAssayInfoService.saveBatchGoodsAssayInfo(assayInfo);
            }
            //添加生产原料投入品
            if(inputsFormList!=null){
                for (GoodsProInputsForm bean:inputsFormList) {
                    bean.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                    bean.setFkGoodsBatchGuid(batchInfo.getGuid());
                    bean.setInputsTime(new Date());
                    bean.setState(1);
                    bean.setCreateTime(new Date());
                    bean.setCreateUser(batchInfoForm.getCreateUser());
                    bean.setModifiedUser(batchInfoForm.getModifiedUser());
                    bean.setModifiedTime(new Date());
                }
                goodsProInputsService.batchSaveProInputs(inputsFormList);
            }
            //添加生产过程信息
            fillProNormalInfo(materialList,batchInfo.getGuid(),batchInfoForm);
            //添加加工过程信息
            fillProNormalInfo(productList,batchInfo.getGuid(),batchInfoForm);
            //添加物联监测信息
            //batchInfoForm.setGuid(batchInfo.getGuid());
            //saveOrUpdateInterAux(batchInfoForm);
            
            return i;
        }
        return 0;
    }

    /**
     * 填充流通生产过程信息
     * @param list
     * @param batchId
     * @param batchInfoForm
     * @throws RuntimeException
     */
    private void fillProNormalInfo(List<GoodsProNormalForm> list,String batchId,GoodsBatchInfoForm batchInfoForm) throws RuntimeException{
        if(list!=null && list.size()>0){
            for (GoodsProNormalForm bean :list) {
                bean.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                bean.setFkGoodsBatchGuid(batchId);
                bean.setState(1);
                bean.setCreateTime(new Date());
                bean.setCreateUser(batchInfoForm.getCreateUser());
                bean.setModifiedUser(batchInfoForm.getModifiedUser());
                bean.setModifiedTime(new Date());
            }
            goodsProNormalService.batchSaveProNormal(list);
        }
    }


    /**
     * 编辑商品批次信息
     *
     * @param batchInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int editGoodsBatchInfo(GoodsBatchInfoForm batchInfoForm) {
        if(null!=batchInfoForm){
            GoodsBatchInfo batchInfo = new GoodsBatchInfo();
            BeanCopyUtil.copyBean(batchInfoForm,batchInfo);
            int i = goodsBatchInfoMapper.updateByPrimaryKeySelective(batchInfo);
            //添加或删除产品标签
            saveOrUpdateGoodsBatchLabel(batchInfoForm);
            
            List<GoodsInfoAuxiliaryForm> auxList = batchInfoForm.getAuxList();
            GoodsAssayInfoForm goodsAssayInfoForm = batchInfoForm.getAssayInfoForm();
            if(null!=auxList && auxList.size()>0){
                for (GoodsInfoAuxiliaryForm bean:auxList) {
                    bean.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                    bean.setFkAuxItmeId(batchInfoForm.getGuid());
                    bean.setFkGoodsGuid(batchInfoForm.getFkGoodsGuid());
                    bean.setState(batchInfoForm.getState());
                    bean.setCreateUser(batchInfoForm.getCreateUser());
                    bean.setCreateTime(new Date());
                    bean.setModifiedUser(batchInfoForm.getModifiedUser());
                    bean.setModifiedTime(new Date());
                }
                goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(batchInfo.getGuid());
                goodsInfoAuxiliaryMapper.bathSaveAuxiliaryInfo(auxList);
            }
            if(goodsAssayInfoForm!=null){
                goodsAssayInfoForm.setState(batchInfoForm.getState());
                goodsAssayInfoForm.setModifiedUser(batchInfoForm.getModifiedUser());
                goodsAssayInfoForm.setModifiedTime(batchInfoForm.getModifiedTime());
                if(goodsAssayInfoForm.getGuid()!=null){
                    goodsAssayInfoService.editBatchGoodsAssayInfo(goodsAssayInfoForm);
                }else{
                    goodsAssayInfoForm.setFkGoodsBatchId(batchInfo.getGuid());
                    goodsAssayInfoForm.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
                    goodsAssayInfoForm.setCreateUser(batchInfoForm.getCreateUser());
                    goodsAssayInfoForm.setCreateTime(batchInfoForm.getCreateTime());
                    goodsAssayInfoService.saveBatchGoodsAssayInfo(goodsAssayInfoForm);
                }
            }
            //添加物联监测信息
            //saveOrUpdateInterAux(batchInfoForm);
            
            return i;
        }
        return 0;
    }

    /**
     * 删除商品批次信息
     * @param batchInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int deleteGoodsBatchInfo(GoodsBatchInfoForm batchInfoForm) {
        if(null!=batchInfoForm && !"".equals(batchInfoForm.getGuid())){
            String batchId = batchInfoForm.getGuid();
            GoodsBatchInfo batchInfo = new GoodsBatchInfo();
            BeanCopyUtil.copyBean(batchInfoForm,batchInfo);
            batchInfo.setState(0);
            batchInfo.setModifiedTime(new Date());
            //更新产品批次信息
            int i = goodsBatchInfoMapper.updateByPrimaryKeySelective(batchInfo);
            //删除商品批次辅助信息
            goodsInfoAuxiliaryMapper.delGoodsFieldAuxiliaryinfoByFkId(batchId);
            //删除检验信息
            goodsAssayInfoService.delBatchGoodsAssayInfoByBatchId(batchId);
            //删除溯源码生成记录信息
            goodsQRCodeRecordService.delGoodsQRCodeRecordByBatchId(batchId);
            //删除原料投入品信息
            goodsProInputsService.delProInputs(batchId);
            //删除流通生产过程信息
            goodsProNormalService.delProNormalByFK(batchId);
            //删除产品标签
            goodsBatchLabelMapper.deleteByBatchGuid(batchId);
            //删除物联监测信息
            //goodsInterService.deleteByBatchGuid(batchId);
            
            return i;
        }
        return 0;
    }

    /**
     * 查询商品批次明细信息
     *
     * @param goodsBatchId
     * @return
     */
    @Override
    public GoodsBatchInfoForm searchGoodsBatchInfoById(String goodsBatchId) {
        GoodsBatchInfoForm batchInfoForm = new GoodsBatchInfoForm();
        GoodsBatchInfo batchInfo = goodsBatchInfoMapper.selectByPrimaryKey(goodsBatchId);
        if(null!=batchInfo){
        	batchInfoForm.setFkGoodsBatchGuid(goodsBatchId);
            String imgStr = batchInfo.getGoodsBatchImages();
            List<String> list = ConvertStringToList.getListFormSplitString(imgStr,",");
            List<GoodsInfoAuxiliary> auxList = goodsInfoAuxiliaryMapper.queryListByParentId(goodsBatchId);
            BeanCopyUtil.copyBean(batchInfo,batchInfoForm);
            batchInfoForm.setImagesList(list);
            //获取辅助信息列表
            if(null!=auxList && auxList.size()>0){
                List<GoodsInfoAuxiliaryForm> auxFormList = BeanCopyUtil.convertList(auxList,GoodsInfoAuxiliaryForm.class);
                batchInfoForm.setAuxList(auxFormList);
            }
            GoodsAssayInfoForm assayInfoForm = goodsAssayInfoService.searchByGoodsBatchId(goodsBatchId);
            if(assayInfoForm!=null){
                batchInfoForm.setAssayInfoForm(assayInfoForm);
            }
            //添加生产原料投入品
            List<GoodsProInputsForm> proInputsList = goodsProInputsService.searchProInputsByBatchGuid(goodsBatchId);
            if(null!=proInputsList && proInputsList.size()>0){
                batchInfoForm.setInputsFormList(proInputsList);
            }
            //添加通用生产过程信息
           List<GoodsProNormalForm> sourceList = goodsProNormalService.searchProNormalByFK(goodsBatchId,PROCESS_SOURCE_TYPE);
           if(null!=sourceList && sourceList.size()>0){
               batchInfoForm.setMaterialList(sourceList);
           }
           //添加通用加工流通过程信息
           List<GoodsProNormalForm> productList = goodsProNormalService.searchProNormalByFK(goodsBatchId,PROCESS_PRODUCT_TYPE);
           if(null!=productList && productList.size()>0){
               batchInfoForm.setProductList(productList);
           }
           //添加产品标签
           List<GoodsLabel> labelList = goodsLabelMapper.selectListBygoodsBatchGuid(goodsBatchId);
           if(null!=labelList && labelList.size()>0){
               batchInfoForm.setLabelList(labelList);
               String labelNames = goodsLabelMapper.selectLabelNamesBygoodsBatchGuid(goodsBatchId);
               batchInfoForm.setLabelNames(labelNames);
           }
           //添加物联监测信息
           List<GoodsInterAuxiliary> interAuxList = goodsInterAuxiliaryService.selectGoodsInterAux(batchInfoForm.getFkFieldGuid());
           batchInfoForm.setInterAuxList(interAuxList);
           
           return batchInfoForm;
        }
        return null;
    }

    /**
     * 添加评价及评级信息
     *
     * @param batchInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveAllFeedbackInfo(GoodsBatchInfoForm batchInfoForm) {
        GoodsBatchFeedbackForm feedbackForm = batchInfoForm.getFeedbackForm();
        GoodsBatchGradeForm gradeForm = batchInfoForm.getGradeForm();
        if(gradeForm!=null && gradeForm.getGradeUser()!=null){
            gradeForm.setState(1);
            gradeForm.setCreateTime(new Date());
            gradeForm.setModifiedTime(new Date());
            feedbackForm.setGradeLevel(gradeForm.getGradeStart());
            int j =goodsBatchGradeService.saveSelective(gradeForm);
            if(j==0){throw new RuntimeException();}
        }
        if(feedbackForm!=null && feedbackForm.getFeedbackUser()!=null){
            String batchId = feedbackForm.getFkGoodsBatchGuid();
            String userId = feedbackForm.getFeedbackUser();
            List<GoodsBatchGradeForm> gradeList = goodsBatchGradeService.getBatchGradeInfoByCond(batchId,userId);
            if(null!=gradeList && gradeList.size()>0){
                GoodsBatchGradeForm tmp = gradeList.get(0);
                feedbackForm.setGradeLevel(tmp.getGradeStart());
            }
            feedbackForm.setState(1);
            feedbackForm.setCreateTime(new Date());
            feedbackForm.setModifiedTime(new Date());
            int i = goodsBatchFeedbackService.saveSelective(feedbackForm);
            if(i==0){throw new RuntimeException();}
        }
        return 1;
    }
    
    /**
     * 添加或删除产品标签
     * @author lym
     */
    private void saveOrUpdateGoodsBatchLabel(GoodsBatchInfoForm batchInfoForm) {
    	log.info("添加产品标签");
    	String labelNames = batchInfoForm.getLabelNames();
    	List<String> labelIds = new ArrayList<>();
    	
    	if(StringUtil.isNotNull(labelNames)) {
    		//List<GoodsLabel> labelList = batchInfoForm.getLabelList();
    		List<String> labelList = ConvertStringToList.getListFormSplitString(labelNames, ",");
			for (String labelName: labelList) {
				GoodsLabel goodLabel = new GoodsLabel();
				goodLabel.setLabelName(labelName);
				//标签是否存在
				GoodsLabel label = goodsLabelMapper.selectGoodsLabelByLabelName(goodLabel.getLabelName());
				if(label != null) {//存在
					//是否已关联产品
					GoodsBatchLabel goodsBatchLabel = new GoodsBatchLabel();
					goodsBatchLabel.setFkGoodsBatchGuid(batchInfoForm.getGuid());
					goodsBatchLabel.setFkGoodLabelGuid(label.getGuid());
					goodsBatchLabel.setState(1);
					
					List<GoodsBatchLabel> list = goodsBatchLabelMapper.selectGoodsBatchLabelList(goodsBatchLabel);
					if(list != null && list.size() > 0) { //存在
						labelIds.add(list.get(0).getGuid());
					}else {//添加产品标签
						goodsBatchLabel.setGuid(GetUUID.getUUID());
						goodsBatchLabel.setCreateUser(batchInfoForm.getCreateUser());
						goodsBatchLabel.setCreateTime(new Date());
						int i = goodsBatchLabelMapper.insertSelective(goodsBatchLabel);
						if (i < 1) {
							log.info("添加产品标签");
							throw new GlobalException(ResultEnum.FAILURE.getCode(), ResultEnum.FAILURE.getMsg());
						}
						labelIds.add(goodsBatchLabel.getGuid());
					}
				}else {//不存在，添加标签
					goodLabel.setGuid(GetUUID.getUUID());  
					goodLabel.setCreateUser(batchInfoForm.getCreateUser());
					goodLabel.setCreateTime(new Date());
					goodLabel.setState(1);
					goodsLabelMapper.insertSelective(goodLabel);
					//添加产品标签
					GoodsBatchLabel goodsBatchLabel = new GoodsBatchLabel();
					goodsBatchLabel.setFkGoodsBatchGuid(batchInfoForm.getGuid());
					goodsBatchLabel.setFkGoodLabelGuid(goodLabel.getGuid());
					goodsBatchLabel.setState(1);
					goodsBatchLabel.setGuid(GetUUID.getUUID());
					goodsBatchLabel.setCreateUser(batchInfoForm.getCreateUser());
					goodsBatchLabel.setCreateTime(new Date());
					
					int i= goodsBatchLabelMapper.insertSelective(goodsBatchLabel);
					if (i < 1) {
						log.info("添加产品标签");
						throw new GlobalException(ResultEnum.FAILURE.getCode(), ResultEnum.FAILURE.getMsg());
					}
					labelIds.add(goodsBatchLabel.getGuid());
				}
			}
			
			//删除产品标签
			goodsBatchLabelMapper.deleteGoodsBatchLabelByNotIds(batchInfoForm.getGuid(), labelIds);
		}else {//无产品标签，删除产品标签
			goodsBatchLabelMapper.deleteGoodsBatchLabelByNotIds(batchInfoForm.getGuid(), null);
		}
    }
    
    /**
     * 添加后修改物联监测信息
     * @author lym
     */
    /*private void saveOrUpdateInterInfo(GoodsBatchInfoForm batchInfoForm) {
    	GoodsInterInfoForm goodsInterInfoForm = batchInfoForm.getInterInfoForm();
        if(goodsInterInfoForm != null) {
        	goodsInterInfoForm.setFkOrgGuid(batchInfoForm.getFkOrgGuid());
    		goodsInterInfoForm.setFkGoodsGuid(batchInfoForm.getFkGoodsGuid());
    		goodsInterInfoForm.setFkGoodsBatchGuid(batchInfoForm.getGuid());
    		goodsInterInfoForm.setState(batchInfoForm.getState());
    		
        	if(goodsInterInfoForm.getGuid() != null) {
        		goodsInterInfoForm.setModifiedUser(batchInfoForm.getModifiedUser());
        		goodsInterInfoForm.setModifiedTime(new Date());
        		goodsInterService.edit(goodsInterInfoForm);
        	}else {
        		goodsInterInfoForm.setCreateUser(batchInfoForm.getCreateUser());
        		goodsInterInfoForm.setCreateTime(new Date());
        		goodsInterService.add(goodsInterInfoForm);
        	}
        }
    }*/
    
    /**
     * 添加后修改物联监测辅助信息
     * @author lym
     */
    /*private void saveOrUpdateInterAux(GoodsBatchInfoForm form) {
    	//删除商品批次物联监测辅助信息
		goodsInterAuxiliaryMapper.deleteByBatchGuid(form.getGuid());
    	
    	List<GoodsInterAuxiliary> auxList = form.getInterAuxList();
        if(auxList != null && auxList.size() > 0) {
        	for (GoodsInterAuxiliary bean : auxList) {
				bean.setFkOrgGuid(form.getFkOrgGuid());
                bean.setFkGoodsGuid(form.getFkGoodsGuid());  
                bean.setFkGoodsBatchGuid(form.getFkGoodsBatchGuid());
                bean.setState(form.getState());
                bean.setCreateUser(form.getCreateUser());
                bean.setCreateTime(form.getCreateTime());
                bean.setModifiedUser(form.getModifiedUser());
                bean.setModifiedTime(form.getModifiedTime());
			}
			//添加物联监测辅助信息
			int i = goodsInterAuxiliaryMapper.batchSave(auxList);
			if(i < 1) {
				throw new GlobalException(ResultEnum.FAILURE.getCode(), "添加物联监测辅助信息失败");
			}
        }
    }*/
}
