package com.hcloud.microservice.goods.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microservice.goods.dao.GoodsProInputsMapper;
import com.hcloud.microservice.goods.facade.entity.GoodsProInputs;
import com.hcloud.microservice.goods.facade.forms.GoodsProInputsForm;
import com.hcloud.microservice.goods.service.GoodsProInputsService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author JYJ
 */
@Service("goodsProInputsService")
public class GoodsProInputsServiceImpl implements GoodsProInputsService {


    @Resource
    private GoodsProInputsMapper goodsProInputsMapper;

    /**
     * 分页查询投入品信息
     *
     * @param inputsForm
     * @return
     */
    @Override
    public PageInfo searchProInputsByPage(GoodsProInputsForm inputsForm) {
        PageHelper.startPage(inputsForm.getPageNo(),inputsForm.getPageSize());
        List<GoodsProInputs> resultList = goodsProInputsMapper.searchProInputsByPage(new PageBean<>(inputsForm));
        List<GoodsProInputsForm> list = BeanCopyUtil.convertList(resultList,GoodsProInputsForm.class);
        return new PageInfo(list);
    }

    /**
     * 查询投入品信息通过商品批次GUID
     *
     * @param batchGuid
     * @return
     */
    @Override
    public List<GoodsProInputsForm> searchProInputsByBatchGuid(String batchGuid) {
        List<GoodsProInputs> resultList = goodsProInputsMapper.searchProInputsByBatchGuid(batchGuid);
        List<GoodsProInputsForm> list = BeanCopyUtil.convertList(resultList,GoodsProInputsForm.class);
        return list;
    }

    /**
     * 批量保存生产原料信息
     *
     * @param list
     * @return
     */
    @Override
    public int batchSaveProInputs(List<GoodsProInputsForm> list) {
        if(null!=list && list.size()>0){
            List<GoodsProInputs> resultList = BeanCopyUtil.convertList(list,GoodsProInputs.class);
            for (GoodsProInputs bean:resultList) {
                String processName = bean.getInputsProcessName();
                String inputDoUser = bean.getInputDoUser();
                String inputProviderUser = bean.getInputProviderUser();
                bean.setInputsProcessName(processName==null ? "":processName);
                bean.setInputDoUser(inputDoUser==null ? "":inputDoUser);
                bean.setInputProviderUser(inputProviderUser==null ? "":inputProviderUser);
            }
           return goodsProInputsMapper.batchSaveProInputs(resultList);
        }
        return 0;
    }

    /**
     * 删除原料投入品信息
     *
     * @param batchGuid
     * @return
     */
    @Override
    public int delProInputs(String batchGuid) {
        return goodsProInputsMapper.delProInputsByFK(batchGuid);
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsProInputsMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsProInputsForm record) {
        GoodsProInputs goodsProInputs = new GoodsProInputs();
        BeanCopyUtil.copyBean(record,goodsProInputs);
        goodsProInputs.setGuid(UUIDUtils.randomUUID());
        return goodsProInputsMapper.insert(goodsProInputs);
    }

    @Override
    public int saveSelective(GoodsProInputsForm record) {
        GoodsProInputs goodsProInputs = new GoodsProInputs();
        BeanCopyUtil.copyBean(record,goodsProInputs);
        goodsProInputs.setGuid(UUIDUtils.randomUUID());
        return goodsProInputsMapper.insertSelective(goodsProInputs);
    }

    @Override
    public GoodsProInputsForm queryByPrimaryKey(String guid) {
        GoodsProInputsForm inputsForm = new GoodsProInputsForm();
        GoodsProInputs proInputs = goodsProInputsMapper.selectByPrimaryKey(guid);
        BeanCopyUtil.copyBean(proInputs,inputsForm);
        return inputsForm;
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsProInputsForm record) {
        GoodsProInputs goodsProInputs = new GoodsProInputs();
        BeanCopyUtil.copyBean(record,goodsProInputs);
        return goodsProInputsMapper.updateByPrimaryKeySelective(goodsProInputs);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsProInputsForm record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsProInputsForm record) {
        GoodsProInputs goodsProInputs = new GoodsProInputs();
        BeanCopyUtil.copyBean(record,goodsProInputs);
        return goodsProInputsMapper.updateByPrimaryKey(goodsProInputs);
    }
}
