package com.hcloud.microservice.goods.service.impl;

import com.hcloud.microservice.goods.dao.GoodsQRCodeRecordMapper;
import com.hcloud.microservice.goods.facade.forms.GoodsQRCodeRecordStatForm;
import com.hcloud.microservice.goods.facade.results.ScanNumStatResult;
import com.hcloud.microservice.goods.facade.results.ScanNumStatisResult;
import com.hcloud.microservice.goods.service.GoodsQRCodeRecordStatService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

@Slf4j
@Service
public class GoodsQRCodeRecordStatServiceImpl implements GoodsQRCodeRecordStatService {

    @Resource
    private GoodsQRCodeRecordMapper goodsQRCodeRecordMapper;

    /**
     * 统计某一年12个月的扫码数量
     *
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    @Override
    public LinkedList<ScanNumStatisResult> statOneYearAllMonthData(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        //ScanNumStatResult result = this.goodsQRCodeRecordMapper.statQRCodeScanNumForOneYeaAllMonth(goodsQRCodeRecordStatForm);
        List<ScanNumStatisResult> list = goodsQRCodeRecordMapper.statQRCodeScanNumByMonth(goodsQRCodeRecordStatForm);
        LinkedList<ScanNumStatisResult> result = new LinkedList();
        for (int i = 1; i <= 12; i++) {
        	String index = String.valueOf(i);
        	ScanNumStatisResult obj = new ScanNumStatisResult();
        	int num = 0;
        	for (ScanNumStatisResult scanNumStatisResult : list) {
				if(index.equals(scanNumStatisResult.getDate())) {
					num = scanNumStatisResult.getNum();
				}
			}
        	obj.setDate(index);
        	obj.setNum(num);
        	
        	result.add(obj);
		}
        
        return result;
    }

    /**
     * 统计昨天和历史累计扫码人数
     *
     * @param goodsQRCodeRecordStatForm
     * @return
     */
    @Override
    public List<ScanNumStatResult> statYesterdayAndAllTimeData(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        return this.goodsQRCodeRecordMapper.statYesterdayAndAllTime(goodsQRCodeRecordStatForm);
    }

    @Override
    public List<ScanNumStatResult> statOneDayAllHoursData(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordMapper.statOneDayAllHours(goodsQRCodeRecordStatForm);
        // 以下逻辑处理数据库的统计数据若不是24个整点的数据,那么缺失整点数据需要补充上
        if (results.size() < 24) {
            List<ScanNumStatResult> finalResults = setZeroToResult(results, 24, 0);

            return finalResults;
        }
        return results;
    }

    @Override
    public List<ScanNumStatResult> statOneWeekSevenDaysData(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        List<ScanNumStatResult> results = this.goodsQRCodeRecordMapper.statOneWeekSevenDays(goodsQRCodeRecordStatForm);
        // 如果结果集长度不足7,需要补零
        if (results.size() < 7) {
            return setZeroToResult(results, 7, 1);
        }
        return results;
    }

    @Override
    public List<ScanNumStatResult> statOneMonthAllDaysData(GoodsQRCodeRecordStatForm goodsQRCodeRecordStatForm) {
        Calendar c = Calendar.getInstance();
        int maxDay = c.getActualMaximum(Calendar.DAY_OF_MONTH);
        List<ScanNumStatResult> results = this.goodsQRCodeRecordMapper.statOneMonthAllDays(goodsQRCodeRecordStatForm);
        if (results.size() < maxDay) {
            return setZeroToResult(results, maxDay, 2);
        }
        return results;
    }

    /**
     * 在数据库中统计一天
     * @param results
     * @param length
     * @param mode
     * @return
     */
    private List<ScanNumStatResult> setZeroToResult(List<ScanNumStatResult> results, int length, int mode) {
        List<ScanNumStatResult> finalResults = new ArrayList<>();
        int i = (mode == 2) ? 1 : 0;
        length = (mode == 2) ? length + 1 : length;
        for (; i < length; i++) {
            boolean flag = false;
            for (int j = 0; j < results.size(); j++) {
                int cs = 0;
                switch (mode) {
                    case 0:
                        cs = results.get(j).getHour();
                        break;
                    case 1:
                        cs = results.get(j).getDayOfWeek();
                        break;
                    case 2:
                        cs = results.get(j).getDayOfMonth();
                        break;
                    default:
                        throw new RuntimeException("比许有一个mode值知名依据小时,星期几来补零");

                }
                if (cs == i) {
                    flag = true;
                }
            }
            if (flag == true) {
                continue;
            } else {
                ScanNumStatResult temp = new ScanNumStatResult();
                switch (mode) {
                    case 0:
                        temp.setHour(i);
                        break;
                    case 1:
                        temp.setDayOfWeek(i == 0 ? 7 : i);
                        break;
                    case 2:
                        temp.setDayOfMonth(i);
                        break;
                    default:
                        throw new RuntimeException("补零出错了");

                }
                temp.setScanNum(0);
                finalResults.add(temp);
            }
        }

        finalResults.addAll(results);

        if (mode == 0) {
            finalResults.sort((o1, o2) -> {
                if (o1.getHour() < o2.getHour()) {
                    return -1;
                } else if (o1.getHour() > o2.getHour()) {
                    return 1;
                } else {
                    return 0;
                }
            });
        } else if (mode == 1) {
            finalResults.sort((o1, o2) -> {
                if (o1.getDayOfWeek() < o2.getDayOfWeek()) {
                    return -1;
                } else if (o1.getDayOfWeek() > o2.getDayOfWeek()) {
                    return 1;
                } else {
                    return 0;
                }
            });
        } else if (mode == 2) {
            finalResults.sort((o1, o2) -> {
                if (o1.getDayOfMonth() < o2.getDayOfMonth()) {
                    return -1;
                } else if (o1.getDayOfMonth() > o2.getDayOfMonth()) {
                    return 1;
                } else {
                    return 0;
                }
            });
        }
        return finalResults;
    }
}
